## created 2021-02-01
## last modified 2021-06-23
## by Moritz Lrig 


# step 1: load packages ---------------------------------------------------

## replace with download folder
setwd("D:\\science\\projects\\2020_asellus_review\\zenodo")

if (!require("pacman")) install.packages("pacman")
pkgs<-c("ggplot2","data.table", "scales","cowplot", "pals","stringr", "viridis",
        "countrycode", "wordcloud")
pacman::p_load(char=pkgs,install=T)
cat(paste("Packages loaded:", paste(pkgs,collapse=", ")), fill=T)
rm(pkgs)


# step 2: assemble subject areas ------------------------------------------

## load data
all = fread("asellus_aquaticus_literature_1867-2020.csv")

## construct keyword df
## accession number == unique paper ID, keywords_plus == WOS keywords
sub = all[,c("accession_number","research_areas")]

## wide table (split into columns by separators)
splits <- max(lengths(strsplit(sub$research_areas, "; ")))
sub.w = sub[, paste0(rep("ra",splits), 1:splits) := tstrsplit(research_areas, "; ", fill = "")]
sub.w[,"research_areas"] = NULL

## long table (i.e. multiple rows with one keyword each per publication)
sub.l = melt(sub.w, id="accession_number")
sub.l = sub.l[complete.cases(value),]
sub.l = sub.l[!grepl("^\\s*$", value),]
sub.l[,value := gsub("- Other Topics","", value)]
sub.l[,value := gsub(" \\(provided by Clarivate Analytics\\)","", value)]
setnames(sub.l, c("variable","value"),c("ra_number","research_areas"))
sub.l = sub.l[order(accession_number)]
sub.l[, research_areas := tolower(research_areas)]
sub.l[, research_areas := str_trim(research_areas)]
sub.l = merge(sub.l, all[,c("accession_number","year_published")])
sub.l[, research_areas := tools::toTitleCase(research_areas)]

## aggregate per field
ra_counts = sub.l[year_published<2021 , .N, by=c("research_areas")]
ra_counts = ra_counts[order(-N)]
ra_counts[, research_areas := factor(research_areas, levels=ra_counts$research_areas)]

## aggregate per year
ra_yearly = unique(sub.l[,c("accession_number","year_published")])
ra_yearly = ra_yearly[year_published<2021, .N, by=c("year_published")]
setnames(ra_yearly, "N", "N_yearly")

## aggregate per year per field
ra_counts_yearly = sub.l[year_published<2021 , .N, by=c("research_areas","year_published")]
ra_counts_yearly = ra_counts_yearly[order(year_published)]
ra_counts_yearly = ra_counts_yearly[complete.cases(ra_counts_yearly),]
setnames(ra_counts_yearly, "N", "N_yearly_subject")

## yearly proportions
ra_props_yearly = merge(ra_counts_yearly, ra_yearly, all=T)
ra_props_yearly[, N_sum := sum(N_yearly_subject), by=c("year_published")]
ra_props_yearly[, N_prop := N_yearly_subject/N_sum, by=c("year_published")]
ra_props_yearly[, N_prop_sum := N_prop * N_yearly, by=c("year_published")]


# step 3: plot linechar ---------------------------------------------------



## create plotting df
plot_data = ra_props_yearly[research_areas %in% ra_counts[1:10,]$research_areas,]
plot_data.w = data.table::dcast(plot_data, research_areas~year_published, fill=0, value.var="N_prop_sum")
plot_data = melt(plot_data.w, id=c("research_areas"), value.name = "N", variable.name = "year_published")
plot_data[, research_areas := factor(research_areas, levels=as.character(ra_counts$research_areas[1:10]))]
plot_data[, year_published := as.numeric(as.character(year_published))]


## set colours for top 10 
cols = c("#008856", "#BE0032", "#0067A5", "#F3C300", "#875692", "#8DB600", 
         "#E68FAC", "#C2B280", "#882D17", "#A1CAF1")
col_vector = c(cols, rep("gray",length(ra_counts$research_areas)-10))
names(col_vector) = ra_counts$research_areas

## line chart
p1 =
  ggplot(plot_data) + theme_cowplot() + ylab("Publications per year") + 
  geom_area(data=ra_yearly, aes(x=year_published, y=N_yearly), fill="gray") +
  geom_area(aes(x=year_published, y=N, fill=research_areas)) +
  scale_fill_manual("Top 10 subject areas", values = col_vector, 
                    labels = paste0(seq(1,10),". ", levels(plot_data$research_areas))) +
  scale_x_continuous(breaks=seq(1850,2020,10)) +
  guides(fill=guide_legend(ncol=2)) +
  theme(
    legend.position = c(0.05,0.6),
    legend.title = element_text(size=12),
    legend.text = element_text(size=10),
    axis.title.x = element_blank(),
    axis.title.y = element_text(margin = margin(t = 0, r = 21, b = 0, l = -9)),
    plot.margin = unit(c(1,2,1.3,1.3), "cm")
  )

## bar chart
p2 =
  ggplot(ra_counts[N>10,]) + theme_cowplot() + ylab("Total number of publications") +
  geom_bar(aes(x=research_areas, y=N, fill=research_areas), stat="identity") +
  scale_fill_manual("Topics",values=col_vector, guide=FALSE) +
  scale_y_continuous(breaks=c(0, 100, 200, 300, 400, 500, 600)) +
  theme(
    axis.text.x = element_text(angle = 300, vjust=1, hjust=0, size=10),
    axis.title.x = element_blank(),
    axis.title.y = element_text(margin = margin(t = 0, r = 13, b = 0, l = 0)),
    plot.margin = unit(c(1,2,1,1), "cm")
  )

## combine
png("fig1a.png", width = 10, height = 10, units = 'in', res = 600)
cowplot::plot_grid(p1,p2, ncol=1, rel_heights = c(0.4,0.6))
dev.off()


# step 4: assemble keywords -----------------------------------------------

## construct keyword df
## accession number == unique paper ID, keywords_plus == WOS keywords
kw = all[,c("accession_number","author_keywords","keywords_plus")]
kw[, keywords := paste0(author_keywords,";", keywords_plus)]
kw = kw[,c("accession_number", "keywords")]
kw[keywords==";", keywords := NA ]

## wide table (split into columns by separators)
splits <- max(lengths(strsplit(kw$keywords, "\\[|\\]|\\/|;|\\.")))
kw.w = kw[, paste0(rep("kw",splits), 1:splits) := tstrsplit(keywords, "\\[|\\]|\\/|\\;|\\.", fill = "")]
kw.w[,"keywords"] = NULL

## long table (i.e. multiple rows with one keyword each per publication)
kw.l = melt(kw.w, id="accession_number")
kw.l = kw.l[complete.cases(value),]
kw.l = kw.l[!grepl("^\\s*$", value),] ## removes whitespace-only entries
setnames(kw.l, c("variable","value"),c("keyword_number","keyword"))
kw.l = kw.l[order(accession_number)]
kw.l[, keyword := tolower(keyword)]


# step 5: plot wordcloud --------------------------------------------------


## make copy
kw_filter = copy(kw.l)

## create list of countries, and filter them out below
countries = data.table(codelist)
countries = tolower(countries$country.name.en)

## keyword editing
kw_filter = kw_filter[, keyword := str_trim(keyword)]
kw_filter = kw_filter[, keyword := str_replace_all(keyword," ","-")]
kw_filter = kw_filter[!keyword %like% "asellus|aquaticus|zoology|crustac|aquatic", ]
kw_filter = kw_filter[!keyword %in% c("p","fig","figs","1", "occ","n","and","c","l","in") , ]
kw_filter = kw_filter[!keyword %in% c(countries, "england", "baltic-sea") , ]

## keyword aggregation
kw_filter_ranks = kw_filter[,.N, by="keyword"]
kw_filter_ranks = kw_filter_ranks[order(-N),]
kw_filter_ranks = kw_filter_ranks[1:75,]

set.seed(22)
png("fig1b.png", width = 6, height = 2.2, units = 'in', res = 600)
wordcloud(kw_filter_ranks$keyword, kw_filter_ranks$N,
          random.order=FALSE, rot.per=0, scale=c(2,0.5),
          use.r.layout=FALSE, colors = viridis(10), fixed.asp = FALSE)
dev.off()



